.actions-dialog {
    /* min-width: 30em;
    max-width: 60em; */
}


.actions-dialog .example {
    font-family: Consolas, monospace;
}

.actions-dialog .preview {
    font-family: Consolas, monospace;
}

.actions-dialog .output {
    font-family: Consolas, monospace;
    line-height: normal;
    overflow: auto;
    height: 300px;
    width: 100%;
    border: 1px var(--jsa-dialog-bd-color);
    padding: var(--jsa-padding);
    border: 1px var(--jsa-dialog-bd-style) var(--jsa-dialog-bd-color);
    border-radius: var(--jsa-dialog-bd-radius);
    margin-bottom: var(--jsa-padding);
}

.actions-dialog .output > div {
    max-width: 100%;
    overflow-wrap: break-word; /*prevent to long lines*/
}

.actions-dialog .output .action-success {
    color: var(--jsa-color-success);
}

.actions-dialog .output .action-warning {
    color: var(--jsa-color-warning);
}

.actions-dialog .output .action-error {
    color: var(--jsa-color-error);
}

.actions-dialog .output .action-info {
    color: var(--jsa-color-info);
}

.progress {
    margin-top: var(--jsa-padding);
}

.actions-dialog .error {
    color: var(--jsa-color-error);
    background-color: var(--jsa-bg-color-error);
    padding: var(--jsa-padding);
    border-radius: var(--jsa-dialog-bd-radius);
    margin-top: var(--jsa-padding);
}

.actions-dialog .success {
    color: var(--jsa-color-success);
    background-color: var(--jsa-bg-color-success);
    padding: var(--jsa-padding);
    border-radius: var(--jsa-dialog-bd-radius);
    margin-top: var(--jsa-padding);
}

.actions-dialog .result {
    color: var(--jsa-color-info);
    border: 1px solid var(--jsa-bg-color-info);
    padding: var(--jsa-padding);
    border-radius: var(--jsa-dialog-bd-radius);
    margin-top: var(--jsa-padding);
}

.action-arguments-table {
    width:100%;
}

.action-arguments-table .name {
    text-align: right;
    vertical-align: middle;
    padding-right: 1rem;
    color: var(--jsa-dialog-color);
}

.action-arguments-table .description {
    font-size: 0.8rem;
    text-align: left;
    vertical-align: middle;
    padding-bottom: 0.5rem;
    color: var(--jsa-dialog-color);
}


.action-arguments-table .value {
    text-align: left;
}

