:root {
    
}

.tool-domain-status {
    background-image: url(../img/tool-domain-status.svg);
}

.tool-domain-status-up {
    background-image: url(../img/tool-domain-status-up.svg);
}

.tool-domain-status-down {
    background-image: url(../img/tool-domain-status-up.svg);
}

.tool-domain-status-up-down {
    background-image: url(../img/tool-domain-status-up-down.svg);
}

.tool-domain-status-info-bubble {
    width: 5rem;
    height: 24rem;
}

.workspace-sticky {
    height: 30%;
}

.palette-sticky {
    height: 30%;
}

.outline-sticky {
    height: 20%;
}

.outline-sticky .outline-sticky-container {
    overflow: hidden;
}

.view-manager-sticky {
    height: 3rem;
}


.graph-view .jsa-view-header{
    background-color: darkblue;
}


.graph-view-container .graph-view-palette {
    /*unset any defined styles*/
}

.graph-view-container .graph-view-outline {
    width: 100%;
    height: 100%;
}

.no-margin {
    margin: 0;
}

div.jsa-app-tooltip {
    background-color: var(--jsa-bd-color);
}

div.jsa-app-tooltip .jsa-bubble-arrow:after {
	background-color: var(--jsa-bd-color);
}



