// This is the baseclass for all plugins that would like to provide a propertiesView pane
// 2020 Bjoern Annighoefer

var PROPERTIES_VIEW = PROPERTIES_VIEW || {};

Object.assign(PROPERTIES_VIEW,(function(){
    
    return {
        //PropertiesPaneProvider : PropertiesPaneProvider
    }
})());


