.ctxtEcore-inline-img {
    display: inline;
    margin: 0 0.125em;
    padding: 0;
    vertical-align: baseline;
    max-inline-size: 16px;
  }

  .context-menu-disabled .ctxtEcore-inline-img{
    display: inline;
    margin: 0 0.125em;
    padding: 0;
    vertical-align: baseline;
    filter: grayscale(100%); 
    -webkit-filter: grayscale(100%); 
    opacity:0.5;
    max-inline-size: 16px;
  }