.editorLoader {
    border: 8px solid #f3f3f3; 
    border-top: 8px solid var(--jsa-color-excited); 
    border-radius: 50%;
    width: var(--jsa-icon-w-l);
    height: var(--jsa-icon-w-l);
    animation: spin 1s linear infinite;
    margin: 0 auto;
    position: absolute;
    top:50%;
    left:50%;
    transform: translate(50%, 50%);
  }
  
  @keyframes spin {
    0% { transform: rotate(0deg); }
    100% { transform: rotate(360deg); }
  }

  .graph-view-container {
    height: 20px; /*must be set! Otherwise mxGraph does not initialize*/
  }