
.canvas{
	position:relative;
	overflow:hidden;
	width:100%;
	height:100%;
	flex-grow: 1;
	background-color: #FFFFFF;
	cursor:default;
}

.editor{
	position:absolute;
	overflow:hidden;
	width: 100%;
    height: 100%;
    caret-color: transparent;
	/* left: 0px;
	right: 0px;
	top: 0px;
	bottom: 0px; */
}


.selection{
	color:#11b3ff;
}
