
.start-modeling-dash-existing-section-dirs {
    padding: 0;
    margin: 0;
}

.start-modeling-dash-existing-section-models {
    padding-left: 0;
    margin-left: 0;
}

.start-modeling-dash-existing-new-elements {
    padding-left: 0;
    margin-left: 0;
}

.start-modeling-dash-entry {
    margin: 0;
	padding-left: 1.5rem;
	list-style: none;
	background-repeat: no-repeat;
	background-position: left center;
    background-size: 0.8rem;
}

.start-modeling-dash-parent-entry {
	background-image: url("../img/parent.svg");
    font-weight: bold;
}

.start-modeling-dash-subdir-entry {
	background-image: url("../img/subdir.svg");
    font-weight: bold;
}

.start-modeling-dash-model-entry {
	background-image: url("../img/object.svg");
}

.start-modeling-dash-new-subdir-entry {
    background-image: url("../img/new-subdir.svg");
    font-style: italic;
}

.start-modeling-dash-new-model-entry {
    background-image: url("../img/new-object.svg");
    font-style: italic;
}

.start-modeling-dash-edit-icon {
    width: 0.8rem;
    height: 0.8rem;
    background-size: 100% 100%;
    background-position: 0px 0px; 
    padding-right: 0.8rem; 
    margin-left: 0.8rem; 
    background-image: url("../img/edit.svg");
    cursor: pointer;
}

.start-modeling-dash-delete-icon {
    width: 0.8rem;
    height: 0.8rem;
    background-size: 100% 100%;
    background-position: 0px 0px; 
    padding-right: 0.8rem; 
    margin-left: 0.8rem; 
    background-image: url("../img/delete.svg");
    cursor: pointer;
}