.graph-view-palette
{
    margin:0px;
    overflow:auto;
}


/*
.paletteBox
 {
  display:inline-flex; 
  position:fixed;
  flex-grow: 1;
  right:33px;
  top:155px; 
  overflow-x:hidden;
  overflow-y:hidden;
  justify-content: flex-start;   
}

.paletteSection
{
    display: flex;
    flex-flow: column;
    height: 100%;
    justify-content: space-around;
}


.paletteTable{
    display:table;
    font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;
    color:#fff;  
    padding-top:5px;
    padding-bottom:5px; 
    flex-grow: 1;
}
*/

.palette-category {
    padding: var(--padding);
    border-bottom: 1px solid var(--border-color-normal);    
}

.palette-category-label {
    font-weight: bold;
    font-size:10pt;
}

/*
.paletteItemCell
{	
    flex-grow: 1;
    display: table-cell;
    vertical-align: middle;
    text-align: left;
    cursor:pointer;
    color:#fff;
    white-space: nowrap;
}
*/

.palette-item {	
    vertical-align: middle;
    cursor:pointer;
    font-weight: normal;
    font-size:10pt;
}

.palette-item:hover {	
    background-color: var(--jsa-color-excited-hover);
}

.palette-item-active {	
    background-color: var(--jsa-color-excited-hover);
}

/*
.editorModeActive
{
	background-color:#868e96;
    border-radius:15px;
    color: #000;
}

.paletteItem
{
    font-family:"Helvetica Neue", Helvetica, Arial, sans-serif;
    color:#fff;
}

.paletteIcon
{
	filter: drop-shadow( 3px 3px 2px rgba(0, 0, 0, .7));
    height:10px;
}

.paletteIcon:hover
{
	filter: drop-shadow( 0px 0px 6px rgba(17, 179, 255,.7));
	cursor:pointer;
}

.paletteModeEnabled
{
	background-color:#ccc;
}
*/

.unselectable
{
    -webkit-touch-callout: none; /* iOS Safari */
    -webkit-user-select: none; /* Safari */
    -khtml-user-select: none; /* Konqueror HTML */
    -moz-user-select: none; /* Firefox */
    -ms-user-select: none; /* Internet Explorer/Edge */
     user-select: none; /* Non-prefixed version, currently supported by Chrome and Opera */ 
}