class ClipboardEvaluator{

    constructor(){
        //this is an interface class
    }

    canHandle(item){

    }

    async copy(item){
        
    }

    async cut(item){

    }

    canPaste(target,item,cmd)
    {

    }

    async paste(target,item,cmd){

    }
    
}


export {ClipboardEvaluator}
