/*
#contextMenu {
  width: 300px;
  padding: 20px;
  font-family: sans-serif;
  font-size: 8pt;
  background-color: #bbb;
  border-radius: 0;
  position: absolute;
  display: none;
}

#contextMenu::before {
  content: "";
  width: 12px;
  height: 12px;
  transform: rotate(45deg);
  background-color: #bbb;
  position: absolute;
  left: -6px;
  top: 68px;
} */

.context-menu-list {
  color: var(--menu-color);
  background-color: var(--menu-background-color);
  border: var(--submenu-border);
  box-shadow: none;
  border-radius: var(--menu-border-radius);
}

.context-menu-item {
  color: var(--menu-color);
  background-color: var(--menu-background-color);
}

.context-menu-item:hover {
  color: var(--menu-hover-color);
  background-color: var(--menu-background-color);
}



.canvasSearchResultItem
{
	  font-family:Lucida Sans Unicode,Sans-Serif;
	  color:#fff;
}

.canvasSearchResultItem:hover
{
	  font-family:Lucida Sans Unicode,Sans-Serif;
	  color:#ccc;
	  text-decoration:underline;
}
