.ecore-modify-properties-pane {

}

.ecore-modify-properties-pane .copy-button {
    margin: var(--padding);
}

.ecore-modify-properties-pane .clone-button {
    margin: var(--padding);
}

.ecore-modify-properties-pane .delete-button {
    background-color: #cc3333;
    margin: var(--padding);
}

.ecore-modify-properties-pane .delete-button:hover {
    color: var(--color-exited);
    background-color: #ccc233;
    box-shadow: var(--box-shadow-hover);
}

