.uiCanvasSearch{
    width: 250px;
    position: absolute;
    display: none;
    color:#fff;
    font-size:12pt;
}

.uiCanvasSearchResults
{  
        width: auto;
        position: absolute;
        padding: 20px;
        display: none;
        background-color:#6c757d;
        border-radius: 5px;
        color: #fff;      
}