/* globals */
:root {
    --views-tabbar-tab-border-color: var(--background-color-dark);
    --views-tabbar-tab-border-color-active: var(--border-color-dark);
}

/* Overwrite tab styles*/
.views-tabbar {
    background-color: var(--tab-background-color);
}

.views-tabbar .jsa-tab {
    color: var(--tab-color);
    border: 1px solid var(--views-tabbar-tab-border-color);
    margin: 0px;
    text-align: center;
    flex-grow: 1;
    cursor: pointer;
    overflow: hidden;
}

.views-tabbar .jsa-tab-active {
    border-top: 2px solid var(--views-tabbar-tab-border-color-active);
    border-bottom: 0px;
}

.views-tabbar .jsa-tab:hover {
    border-top: 1px solid var(--views-tabbar-tab-border-color-active);
}

/*new style for the default view manager*/
.views-tabbar-view-manager-mod {
    /*top: calc(var(--menu-h)+var(--tabbar-h));*/
    top: 4.5rem;
}