/* properties bubble */

.properties-bubble {
    height: 20rem;
    padding: 0;
}

/*main pane*/

.propertiesView-main-pane {
    width: 100%;
    height: 100%;
    display: flex;
    flex-direction: column;
    border: inherit;
}


/* headlines */
.properties-view h1 {
    font-size: 1rem;
    font-weight:bold;
}

.properties-view h2 {
    font-size: 0.9rem;
    font-weight:bold;
}

.properties-view h3 {
    font-size: 0.8rem;
    font-weight:bold;
}

/*properties pane tables*/

.properties-view .properties-table {
    width:100%;
}

.properties-view .properties-table .properties-table{
    width:100%;
}

.properties-view .properties-table .feature {
    text-align: right;
    vertical-align: middle;
    padding-right: 1rem;
}

.properties-view .properties-table .value {
    text-align: left;
}

.properties-view .properties-table .description {
    font-size: 0.8rem;
    text-align: left;
    vertical-align: middle;
    padding-bottom: 0.5rem;
}